@@ -40,6 +40,7 @@ static int isprime(const int n)
 }
 
 /* return number of bits set in bitmap (Hamming Weight / popcount) */
+#ifdef RQ_TRACK_COMPONENTS
 inline static unsigned int hamm(const unsigned char *map, size_t len)
 {
 	unsigned int c = 0;
@@ -50,6 +51,7 @@ inline static unsigned int hamm(const unsigned char *map, size_t len)
 #endif
 	return c;
 }
+#endif
 
 /* convert ESI to ISI (5.3.1) */
 inline static uint32_t esi2isi(const rq_t *rq, const uint32_t esi)
@@ -149,9 +151,11 @@ uint8_t *rq_encode_symbol(const rq_t *rq, const matrix_t *C, const uint32_t isi,
 static uint32_t rq_random_esi(uint32_t min)
 {
 	/* NB: ESI is a 24-bit unsigned integer (3.2) */
-	uint32_t esi;
+	uint32_t esi = UINT32_MAX;
 #ifdef HAVE_LIBSODIUM
 	esi = randombytes_uniform(RQ_ESI_MAX - min) + min;
+#elif defined (HAVE_ARC4RANDOM_UNIFORM)
+	esi = arc4random_uniform(RQ_ESI_MAX - min) + min;
 #else
 	/* read 24 bits (3 bytes) from /dev/random */
 	ssize_t byt, len = 3;
@@ -163,14 +167,16 @@ static uint32_t rq_random_esi(uint32_t min)
 	while ((byt = getrandom(a, len, 0)) != len) {
 #else
 	static int f; /* we'll keep the handle until program exit */
-	if (!f) f = open("/dev/urandom", O_RDONLY);
+	if (!f) {
+		f = open("/dev/urandom", O_RDONLY);
+		if (f == -1) return (f = 0), esi;
+	}
 	while ((byt = read(f, a, len)) != len) {
 #endif
 		if (byt == -1) break;
 		a += byt; len -= byt;
 	}
-	esi += min;
-	esi &= RQ_ESI_MAX; /* mask overflow */
+	esi = min + (esi % (RQ_ESI_MAX - min));
 #endif
 	return esi;
 }
@@ -190,6 +196,7 @@ uint8_t *rq_symbol(const rq_t *rq, rq_pid_t *pid, uint8_t *sym, int flags)
 	matrix_t C = rq_matrix_C_by_SBN(rq, sbn);
 	if ((flags & RQ_RAND) == RQ_RAND) {
 		esi = rq_random_esi(rq->K);
+		if (esi == UINT32_MAX) return NULL;
 		*pid = rq_pidsetesi(*pid, esi);
 	}
 	else esi = rq_pid2esi(*pid);
@@ -514,7 +521,9 @@ int rq_decode(rq_t *rq, uint8_t *dec, uint8_t *enc, uint32_t ESI[], uint32_t nes
 	matrix_free(&D);
 	matrix_new(&Cm, rq->L, rq->T, C, 0);
 	for (int esi = 0; esi < rq->K; esi++) {
-		rq_encode_symbol(rq, &Cm, esi, dec + rq->T * esi);
+		if (!rq_encode_symbol(rq, &Cm, esi, dec + rq->T * esi)) {
+			rc = -1; break;
+		}
 	}
 	free(C);
 fail:
